/*
 *  Copyright (C) 2010-2020. Mellanox Technologies, Ltd. ALL RIGHTS RESERVED.
 *
 *    Licensed under the Apache License, Version 2.0 (the "License"); you may
 *    not use this file except in compliance with the License. You may obtain
 *    a copy of the License at http://www.apache.org/licenses/LICENSE-2.0
 *
 *    THIS CODE IS PROVIDED ON AN  *AS IS* BASIS, WITHOUT WARRANTIES OR
 *    CONDITIONS OF ANY KIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT
 *    LIMITATION ANY IMPLIED WARRANTIES OR CONDITIONS OF TITLE, FITNESS
 *    FOR A PARTICULAR PURPOSE, MERCHANTABLITY OR NON-INFRINGEMENT.
 *
 *    See the Apache Version 2.0 License for specific language governing
 *    permissions and limitations under the License.
 *
 */

#ifndef __SX_DBG_H__
#define __SX_DBG_H__

#include <stdlib.h>
#include <stdint.h>

#include "sx/sdk/auto_headers/sx_dbg_auto.h"


/************************************************
 *  Defines
 ***********************************************/

#define SX_GENERATE_ENUM(ENUM, STR) ENUM,

/**
 * Maximum dump file path length.
 */
#define SX_API_DUMP_PATH_LEN_LIMIT     256
#define SX_API_DUMP_EXT_FILE_LEN_LIMIT (66)
#define SX_API_DUMP_EXT_PATH_LEN_LIMIT (SX_API_DUMP_PATH_LEN_LIMIT - SX_API_DUMP_EXT_FILE_LEN_LIMIT)

/************************************************
 *  Type definitions
 ***********************************************/

/* TODO - currently - this API are using a workaround in order to solve compilation errors
 * when using SWIG to generate python modules.
 * We don't use the types in atcam_types.h but rather we use native types defined by the standard.
 * Obviously, this should be changes as soon as possible.
 */
typedef uint32_t sx_atcam_region_id_t;
typedef uint8_t sx_atcam_erp_id_t;

typedef enum sx_dbg_atcam_cmd_type {
    SX_DBG_ATCAM_CMD_ERPS_SAMPLING_SET_E,
    SX_DBG_ATCAM_CMD_ERPS_COUNTERS_SAMPLE_E,
    SX_DBG_ATCAM_CMD_REGION_ERPS_GET_E,
    SX_DBG_ATCAM_CMD_REORDER_REGION_ERPS_E,
    SX_DBG_ATCAM_CMD_REGION_COUNTERS_SET_E,
    SX_DBG_ATCAM_CMD_REGION_COUNTERS_READ_E,
    SX_DBG_ATCAM_CMD_ERP_COUNTERS_SET_E,
    SX_DBG_ATCAM_CMD_ERP_COUNTERS_READ_E,
} sx_dbg_atcam_cmd_type_e;

typedef struct sx_dbg_atcam_erps_sampling_set {
    boolean_t enable_sampling;
} sx_dbg_atcam_erps_sampling_set_t;

typedef struct sx_dbg_atcam_region_erps_get {
    uint32_t region_id;
    uint8_t  erps_ids[16];
} sx_dbg_atcam_region_erps_get_t;

typedef struct sx_dbg_atcam_reorder_region_erps {
    uint32_t  region_id;
    uint8_t   erps_ids[16];
    boolean_t completed;
} sx_dbg_atcam_reorder_region_erps_t;

typedef struct sx_dbg_atcam_region_counters_set {
    uint32_t region_id;                 /* The region ids on which to apply the region counters */
    uint32_t region_id_bitmask;         /* 0 - compare region_id relevant bit, 1 - ignore region_id relevant bit */
} sx_dbg_atcam_region_counters_set_t;

typedef struct sx_dbg_atcam_region_counters_read {
    boolean_t clear;                  /* Whether to clear the counters after the read */
    uint64_t  lookup_count;           /* The number of times the regions were accessed */
    uint64_t  full_lookup_count;      /* The number of times the regions performed a full lookup (Not a cache match) */
} sx_dbg_atcam_region_counters_read_t;

typedef struct sx_dbg_atcam_erp_counters_set {
    uint32_t  region_id;             /* The region ids on which to apply the region counters */
    uint32_t  region_id_bitmask;     /* 0 bit - compare region_id relevant bit, 1 bit - ignore region_id relevant bit */
    uint16_t  erp_bitmask;           /* Each 1 bit represent an ERP we're interested in. If region_id_bitmask !=0 all erps are counted */
    boolean_t ctcam;                 /* Count CTCAM lookups . If region_id_bitmask !=0 CTCAM is always counted */
} sx_dbg_atcam_erp_counters_set_t;

typedef struct sx_dbg_atcam_erp_counters_read {
    boolean_t clear;                   /* Whether to clear the counters after the read */
    uint64_t  initial_count;           /* ERPs + CTCAM access count (erp vector) before any filtering */
    uint64_t  post_bf_count;           /* ERPs active after BF. Doesn't increment on CTCAM */
    uint64_t  lookup_count;            /* ERPs + CTCAM lookups after BF and Prune */
    uint64_t  any_match_count;         /* ERPs + CTCAM had any match (may not be the final match) */
    uint64_t  final_match_count;       /* ERPs or CTCAM had a final match */
} sx_dbg_atcam_erp_counters_read_t;

typedef union sx_dbg_atcam_cmd_info_data {
    sx_dbg_atcam_erps_sampling_set_t    erps_sample_set;
    sx_dbg_atcam_region_erps_get_t      region_erps_get;
    sx_dbg_atcam_reorder_region_erps_t  reorder_region_erps;
    sx_dbg_atcam_region_counters_set_t  region_counters_set;
    sx_dbg_atcam_region_counters_read_t region_counters_read;
    sx_dbg_atcam_erp_counters_set_t     erp_counters_set;
    sx_dbg_atcam_erp_counters_read_t    erp_counters_read;
} sx_dbg_atcam_cmd_info_data_t;

typedef struct sx_dbg_atcam_cmd_info {
    sx_dbg_atcam_cmd_type_e      type;
    sx_dbg_atcam_cmd_info_data_t data;
} sx_dbg_atcam_cmd_info_t;

/**
 * sx_dbg_extra_info_t structure is used to store sx_api_dbg_generate_dump_ext
 * function parameters.
 *
 * dev_id          - the requested device id (default 255)
 * is_async        - should the debug info be taken synchronously blocking the SDK
 *                   or asynchronously allowing the SDK to continue with other flows
 * force_db_refresh- should the debug info be from the current SDK cache or rebuilt
 * timeout         - timeout for the debug dump extra API run
 * path            - Path to the directory which will hold the debug info (path only)
 */
typedef struct sx_dbg_extra_info {
    uint8_t   dev_id;
    boolean_t is_async;
    boolean_t force_db_refresh;
    uint32_t  timeout_usec;
    char      path[SX_API_DUMP_EXT_PATH_LEN_LIMIT];
} sx_dbg_extra_info_t;


#define SX_DBG_POLICY_FOREACH_OBJECT_TYPE(SX_DBG_POLICY)                                    \
    SX_DBG_POLICY(SX_DBG_POLICY_NO_AUTO_DEBUG_EXTRACTION_E = 0, "no auto debug extraction") \
    SX_DBG_POLICY(SX_DBG_POLICY_AUTO_DEBUG_EXTRACTION_E = 1, "auto debug extraction")       \

/**
 * sx_dbg_policy_e is used to set the SDK debug policy
 */
typedef enum sx_dbg_policy {
    SX_DBG_POLICY_FOREACH_OBJECT_TYPE(SX_GENERATE_ENUM)
} sx_dbg_policy_e;

/**
 * sx_dbg_extra_info_t structure is used to store sx_api_dbg_control function parameters.
 *
 * dev_id - the requested device id (default 255)
 * arm    - arm the debug for this device
 * policy - should SDK extract debug information independently (sx_api_dbg_generate_dump_ext)
 */
typedef struct sx_dbg_control_params {
    uint8_t         dev_id;
    boolean_t       enable;
    sx_dbg_policy_e policy;
    boolean_t       trigger_test;
} sx_dbg_control_params_t;
#endif /* __SX_DBG_H__ */
